package com.ruoyi.dashboard.domain.revenueStat;

import java.time.LocalDate;
import java.util.List;

import org.springframework.format.annotation.DateTimeFormat;

import com.ruoyi.bst.area.domain.AreaQuery;
import com.ruoyi.bst.areaJoin.domain.AreaJoinQuery;
import com.ruoyi.bst.bonus.domain.BonusQuery;
import com.ruoyi.bst.bonus.domain.enums.BonusStatus;
import com.ruoyi.bst.bonusRefund.domain.BonusRefundQuery;
import com.ruoyi.bst.device.domain.DeviceQuery;
import com.ruoyi.bst.fault.domain.FaultQuery;
import com.ruoyi.bst.mchApply.domain.MchApplyQuery;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.enums.OrderStatus;
import com.ruoyi.bst.withdraw.domain.WithdrawQuery;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

@Data
public class RevenueStatQuery {
    @ApiModelProperty("日期范围")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private List<LocalDate> dateRange;

    @ApiModelProperty("查询数据")
    private List<String> keys;

    @ApiModelProperty("用户ID")
    private Long userId;

    @ApiModelProperty("运营区ID")
    private Long areaId;

    @ApiModelProperty("数据隔离")
    private Boolean scope;
    

    private OrderQuery baseOrderQuery() {
        OrderQuery query = new OrderQuery();
        query.setStatusList(OrderStatus.valid());
        query.setUserId(userId);
        query.setAreaId(areaId);
        query.setScope(scope);
        return query;
    }

    private BonusQuery baseBonusQuery() {
        BonusQuery query = new BonusQuery();
        query.setStatusList(BonusStatus.valid());
        query.setArrivalId(userId);
        query.setAreaId(areaId);
        query.setScope(scope);
        return query;
    }

    private BonusRefundQuery baseBonusRefundQuery() {   
        BonusRefundQuery query = new BonusRefundQuery();
        query.setBonusStatusList(BonusStatus.valid());
        query.setBonusArrivalId(userId);
        query.setBonusAreaId(areaId);
        query.setScope(scope);
        return query;
    }

    public OrderQuery toOrderQuery() {
        OrderQuery query = baseOrderQuery();
        query.setCreateDateRange(dateRange);
        return query;
    }

    public OrderQuery toOrderQueryForEnd() {
        OrderQuery query = baseOrderQuery();
        query.setEndDateRange(dateRange);
        return query;
    }

    public BonusQuery toBonusQuery() {
        BonusQuery query = baseBonusQuery();
        query.setCreateDateRange(dateRange);
        return query;
    }

    public BonusRefundQuery toBonusRefundQuery() {
        BonusRefundQuery query = baseBonusRefundQuery();
        query.setCreateDateRange(dateRange);
        return query;
    }

    public AreaQuery toAreaQuery() {
        AreaQuery query = new AreaQuery();
        query.setCreateDateRange(dateRange);
        query.setId(areaId);
        query.setUserId(userId);
        query.setScope(scope);
        return query;
    }

    public AreaJoinQuery toAreaJoinQuery() {
        AreaJoinQuery query = new AreaJoinQuery();
        query.setCreateDateRange(dateRange);
        query.setAreaId(areaId);
        query.setUserId(userId);
        query.setScope(scope);
        return query;
    }

    public DeviceQuery toDeviceQuery() {
        DeviceQuery query = new DeviceQuery();
        query.setCreateDateRange(dateRange);
        query.setAreaId(areaId);
        query.setMchId(userId);
        query.setScope(scope);
        return query;
    }

    public WithdrawQuery toWithdrawQuery() {
        WithdrawQuery query = new WithdrawQuery();
        query.setCreateDateRange(dateRange);
        query.setUserId(userId);
        query.setScope(scope);
        return query;
    }

    public MchApplyQuery toMchApplyQuery() {
        MchApplyQuery query = new MchApplyQuery();
        query.setCreateDateRange(dateRange);
        query.setUserId(userId);
        query.setScope(scope);
        return query;
    }

    public FaultQuery toFaultQuery() {
        FaultQuery query = new FaultQuery();
        query.setCreateDateRange(dateRange);
        query.setStoreId(userId);
        query.setAreaId(areaId);
        query.setScope(scope);
        return query;
    }
}
